@@ -1,22 +1,18 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import datetime as dt
 import os.path
 from calendar import monthrange
 from functools import lru_cache
+from importlib.resources import is_resource
 from typing import Optional
 
 from hypothesis.errors import InvalidArgument
@@ -27,25 +23,17 @@ from hypothesis.strategies._internal.misc import just, none
 from hypothesis.strategies._internal.strategies import SearchStrategy
 from hypothesis.strategies._internal.utils import defines_strategy
 
-# These standard-library modules are required for the timezones() and
-# timezone_keys() strategies, but not present in older versions of Python.
-# We therefore try to import them here, but only raise errors recommending
-# `pip install hypothesis[zoneinfo]` to install the backports (if needed)
-# when those strategies are actually used.
-try:
-    import importlib.resources as importlib_resources
-except ImportError:
-    try:
-        import importlib_resources  # type: ignore
-    except ImportError:
-        importlib_resources = None  # type: ignore
+# The zoneinfo module, required for the timezones() and timezone_keys()
+# strategies, is new in Python 3.9 and the backport might be missing.
 try:
     import zoneinfo
 except ImportError:
     try:
         from backports import zoneinfo  # type: ignore
     except ImportError:
-        zoneinfo = None
+        # We raise an error recommending `pip install hypothesis[zoneinfo]`
+        # when timezones() or timezone_keys() strategies are actually used.
+        zoneinfo = None  # type: ignore
 
 DATENAMES = ("year", "month", "day")
 TIMENAMES = ("hour", "minute", "second", "microsecond")
@@ -98,7 +86,10 @@ def datetime_does_not_exist(value):
         # 9999, so it should be a very small fraction of possible values.
         return True
 
-    if value.tzinfo.utcoffset != roundtrip.tzinfo.utcoffset:
+    if (
+        value.tzinfo is not roundtrip.tzinfo
+        and value.utcoffset() != roundtrip.utcoffset()
+    ):
         # This only ever occurs during imaginary (i.e. nonexistent) datetimes,
         # and only for pytz timezones which do not follow PEP-495 semantics.
         # (may exclude a few other edge cases, but you should use zoneinfo anyway)
@@ -354,15 +345,10 @@ def _valid_key_cacheable(tzpath, key):
         # This branch is only taken for names which are known to zoneinfo
         # but not present on the filesystem, i.e. on Windows with tzdata,
         # and so is never executed by our coverage tests.
-        if importlib_resources is None:
-            raise ImportError(
-                "The importlib_resources module is required, but could not be "
-                "imported.  Run `pip install hypothesis[zoneinfo]` and try again."
-            )
         *package_loc, resource_name = key.split("/")
         package = "tzdata.zoneinfo." + ".".join(package_loc)
         try:
-            return importlib_resources.is_resource(package, resource_name)
+            return is_resource(package, resource_name)
         except ModuleNotFoundError:
             return False
 
@@ -393,8 +379,10 @@ def timezone_keys(
     .. note::
 
         The :mod:`python:zoneinfo` module is new in Python 3.9, so you will need
-        to install the :pypi:`backports.zoneinfo` module on earlier versions, and
-        the :pypi:`importlib_resources` backport on Python 3.6.
+        to install the :pypi:`backports.zoneinfo` module on earlier versions.
+
+        `On Windows, you will also need to install the tzdata package
+        <https://docs.python.org/3/library/zoneinfo.html#data-sources>`__.
 
         ``pip install hypothesis[zoneinfo]`` will install these conditional
         dependencies if and only if they are needed.
@@ -451,8 +439,10 @@ def timezones(*, no_cache: bool = False) -> SearchStrategy["zoneinfo.ZoneInfo"]:
     .. note::
 
         The :mod:`python:zoneinfo` module is new in Python 3.9, so you will need
-        to install the :pypi:`backports.zoneinfo` module on earlier versions, and
-        the :pypi:`importlib_resources` backport on Python 3.6.
+        to install the :pypi:`backports.zoneinfo` module on earlier versions.
+
+        `On Windows, you will also need to install the tzdata package
+        <https://docs.python.org/3/library/zoneinfo.html#data-sources>`__.
 
         ``pip install hypothesis[zoneinfo]`` will install these conditional
         dependencies if and only if they are needed.
