@@ -28,7 +28,7 @@ from types import FunctionType
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import PYPY, ForwardRef, typing_root_type
+from hypothesis.internal.compat import PYPY
 from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
 from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
@@ -39,16 +39,18 @@ from hypothesis.strategies._internal.ipaddress import (
 from hypothesis.strategies._internal.lazy import unwrap_strategies
 from hypothesis.strategies._internal.strategies import OneOfStrategy
 
+UnionType: typing.Any
+try:
+    # The type of PEP-604 unions (`int | str`), added in Python 3.10
+    from types import UnionType
+except ImportError:
+    UnionType = ()
+
 try:
     import typing_extensions
 except ImportError:
     typing_extensions = None  # type: ignore
 
-try:
-    from typing import GenericMeta as _GenericMeta  # python < 3.7
-except ImportError:
-    _GenericMeta = ()  # type: ignore
-
 try:
     from typing import _GenericAlias  # type: ignore  # python >= 3.7
 except ImportError:
@@ -60,14 +62,49 @@ except ImportError:
     try:
         from typing_extensions import _AnnotatedAlias
     except ImportError:
-        try:
-            from typing_extensions import (  # type: ignore
-                AnnotatedMeta as _AnnotatedAlias,
-            )
+        _AnnotatedAlias = ()
 
-            assert sys.version_info[:2] == (3, 6)
-        except ImportError:
-            _AnnotatedAlias = ()
+TypeAliasTypes: tuple = ()
+try:
+    TypeAliasTypes += (typing.TypeAlias,)
+except AttributeError:
+    pass  # Is missing for `python<3.10`
+try:
+    TypeAliasTypes += (typing_extensions.TypeAlias,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `typing_extensions<3.10`
+
+ClassVarTypes: tuple = (typing.ClassVar,)
+try:
+    ClassVarTypes += (typing_extensions.ClassVar,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+FinalTypes: tuple = ()
+try:
+    FinalTypes += (typing.Final,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    FinalTypes += (typing_extensions.Final,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+# We use this variable to be sure that we are working with a type from `typing`:
+typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
+
+# We use this to disallow all non-runtime types from being registered and resolved.
+# By "non-runtime" we mean: types that do not really exist in python's
+# and are just added for more fancy type annotations.
+# `Final` is a great example: it just indicates
+# that this value can't be reassigned.
+NON_RUNTIME_TYPES = frozenset(
+    (
+        *ClassVarTypes,
+        *TypeAliasTypes,
+        *FinalTypes,
+    )
+)
 
 
 def type_sorting_key(t):
@@ -106,9 +143,7 @@ def try_issubclass(thing, superclass):
             getattr(superclass, "__origin__", None) or superclass,
         ):
             superclass_args = getattr(superclass, "__args__", None)
-            if sys.version_info[:2] == (3, 6) or not superclass_args:
-                # Python 3.6 doesn't have PEP-560 semantics or __orig_bases__,
-                # so there's no point continuing; we therefore stop early.
+            if not superclass_args:
                 # The superclass is not generic, so we're definitely a subclass.
                 return True
             # Sadly this is just some really fiddly logic to handle all the cases
@@ -142,6 +177,14 @@ def is_a_new_type(thing):
     return isinstance(thing, typing.NewType)  # pragma: no cover  # on 3.8, anyway
 
 
+def is_a_union(thing):
+    """Return True if thing is a typing.Union or types.UnionType (in py310)."""
+    return (
+        isinstance(thing, UnionType)
+        or getattr(thing, "__origin__", None) is typing.Union
+    )
+
+
 def is_a_type(thing):
     """Return True if thing is a type or a generic type like thing."""
     return isinstance(thing, type) or is_generic_type(thing) or is_a_new_type(thing)
@@ -152,11 +195,7 @@ def is_typing_literal(thing):
         hasattr(typing, "Literal")
         and getattr(thing, "__origin__", None) == typing.Literal
         or hasattr(typing_extensions, "Literal")
-        and (
-            getattr(thing, "__origin__", None) == typing_extensions.Literal
-            or sys.version_info[:2] == (3, 6)
-            and isinstance(thing, type(typing_extensions.Literal[None]))
-        )
+        and getattr(thing, "__origin__", None) == typing_extensions.Literal
     )
 
 
@@ -185,7 +224,7 @@ def find_annotated_strategy(annotated_type):  # pragma: no cover
 def has_type_arguments(type_):
     """Decides whethere or not this type has applied type arguments."""
     args = getattr(type_, "__args__", None)
-    if args and isinstance(type_, (_GenericAlias, _GenericMeta)):
+    if args and isinstance(type_, _GenericAlias):
         # There are some cases when declared types do already have type arguments
         # Like `Sequence`, that is `_GenericAlias(abc.Sequence[T])[T]`
         parameters = getattr(type_, "__parameters__", None)
@@ -197,10 +236,8 @@ def has_type_arguments(type_):
 def is_generic_type(type_):
     """Decides whether a given type is generic or not."""
     # The ugly truth is that `MyClass`, `MyClass[T]`, and `MyClass[int]` are very different.
-    # In different python versions they might have the same type (3.6)
-    # or it can be regular type vs `_GenericAlias` (3.7+)
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
-    # while the second condition is for `MyClass` in `python3.7+`.
+    # while the second condition is for `MyClass`.
     return isinstance(type_, typing_root_type) or (
         isinstance(type_, type) and typing.Generic in type_.__mro__
     )
@@ -217,14 +254,6 @@ def _try_import_forward_ref(thing, bound):  # pragma: no cover
     try:
         return typing._eval_type(bound, vars(sys.modules[thing.__module__]), None)
     except (KeyError, AttributeError, NameError):
-        if (
-            isinstance(thing, typing.TypeVar)
-            and getattr(thing, "__module__", None) == "typing"
-        ):
-            raise ResolutionFailed(
-                "It looks like you're using a TypeVar bound to a ForwardRef on Python "
-                "3.6, which is not supported - try ugrading to Python 3.7 or later."
-            ) from None
         # We fallback to `ForwardRef` instance, you can register it as a type as well:
         # >>> from typing import ForwardRef
         # >>> from hypothesis import strategies as st
@@ -233,9 +262,8 @@ def _try_import_forward_ref(thing, bound):  # pragma: no cover
 
 
 def from_typing_type(thing):
-    # We start with special-case support for Union and Tuple - the latter
-    # isn't actually a generic type. Then we handle Literal since it doesn't
-    # support `isinstance`.
+    # We start with special-case support for Tuple, which isn't actually a generic
+    # type; then Final, Literal, and Annotated since they don't support `isinstance`.
     #
     # We then explicitly error on non-Generic types, which don't carry enough
     # information to sensibly resolve to strategies at runtime.
@@ -284,11 +312,9 @@ def from_typing_type(thing):
 
     # Some "generic" classes are not generic *in* anything - for example both
     # Hashable and Sized have `__args__ == ()` on Python 3.7 or later.
-    # (In 3.6 they're just aliases for the collections.abc classes)
     origin = getattr(thing, "__origin__", thing)
     if (
-        typing.Hashable is not collections.abc.Hashable
-        and origin in vars(collections.abc).values()
+        origin in vars(collections.abc).values()
         and len(getattr(thing, "__args__", None) or []) == 0
     ):
         return st.from_type(origin)
@@ -305,9 +331,6 @@ def from_typing_type(thing):
         # ItemsView can cause test_lookup.py::test_specialised_collection_types
         # to fail, due to weird isinstance behaviour around the elements.
         mapping.pop(typing.ItemsView, None)
-        if sys.version_info[:2] == (3, 6):  # pragma: no cover
-            # `isinstance(dict().values(), Container) is False` on py36 only -_-
-            mapping.pop(typing.ValuesView, None)
     if typing.Deque in mapping and len(mapping) > 1:
         # Resolving generic sequences to include a deque is more trouble for e.g.
         # the ghostwriter than it's worth, via undefined names in the repr.
@@ -323,7 +346,7 @@ def from_typing_type(thing):
         # if there is more than one allowed type, and the element type is
         # not either `int` or a Union with `int` as one of its elements.
         elem_type = (getattr(thing, "__args__", None) or ["not int"])[0]
-        if getattr(elem_type, "__origin__", None) is typing.Union:
+        if is_a_union(elem_type):
             union_elems = elem_type.__args__
         else:
             union_elems = ()
@@ -334,7 +357,7 @@ def from_typing_type(thing):
             mapping.pop(typing.ByteString, None)
     elif (
         (not mapping)
-        and isinstance(thing, ForwardRef)
+        and isinstance(thing, typing.ForwardRef)
         and thing.__forward_arg__ in vars(builtins)
     ):
         return st.from_type(getattr(builtins, thing.__forward_arg__))
@@ -362,7 +385,7 @@ def can_cast(type, value):
 
 
 def _networks(bits):
-    return st.tuples(st.integers(0, 2 ** bits - 1), st.integers(-bits, 0).map(abs))
+    return st.tuples(st.integers(0, 2**bits - 1), st.integers(-bits, 0).map(abs))
 
 
 utc_offsets = st.builds(
@@ -464,6 +487,8 @@ _global_type_lookup: typing.Dict[
     classmethod: st.builds(classmethod, st.just(lambda self: self)),
     staticmethod: st.builds(staticmethod, st.just(lambda self: self)),
     super: st.builds(super, st.from_type(type)),
+    re.Match: st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool),
+    re.Pattern: st.builds(re.compile, st.sampled_from(["", b""])),
     # Pull requests with more types welcome!
 }
 if zoneinfo is not None:  # pragma: no branch
@@ -490,12 +515,6 @@ else:  # pragma: no cover
 _global_type_lookup[type] = st.sampled_from(
     [type(None)] + sorted(_global_type_lookup, key=str)
 )
-
-if sys.version_info[:2] >= (3, 7):  # pragma: no branch
-    _global_type_lookup[re.Match] = (
-        st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool)
-    )
-    _global_type_lookup[re.Pattern] = st.builds(re.compile, st.sampled_from(["", b""]))
 if sys.version_info[:2] >= (3, 9):  # pragma: no cover
     # subclass of MutableMapping, and in Python 3.9 we resolve to a union
     # which includes this... but we don't actually ever want to build one.
@@ -572,7 +591,7 @@ _global_type_lookup.update(
     }
 )
 if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
-    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()  # type: ignore
+    _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()
 
 
 def register(type_, fallback=None, *, module=typing):
@@ -608,12 +627,12 @@ def resolve_Type(thing):
         # This branch is for Python < 3.8, when __args__ was not always tracked
         return st.just(type)  # pragma: no cover
     args = (thing.__args__[0],)
-    if getattr(args[0], "__origin__", None) is typing.Union:
+    if is_a_union(args[0]):
         args = args[0].__args__
     # Duplicate check from from_type here - only paying when needed.
     args = list(args)
     for i, a in enumerate(args):
-        if type(a) == ForwardRef:
+        if type(a) == typing.ForwardRef:
             try:
                 args[i] = getattr(builtins, a.__forward_arg__)
             except AttributeError:
@@ -793,7 +812,7 @@ def resolve_TypeVar(thing):
 
     if getattr(thing, "__bound__", None) is not None:
         bound = thing.__bound__
-        if isinstance(bound, ForwardRef):
+        if isinstance(bound, typing.ForwardRef):
             bound = _try_import_forward_ref(thing, bound)
         strat = unwrap_strategies(st.from_type(bound))
         if not isinstance(strat, OneOfStrategy):
